"""Base class for Hume clients."""
from abc import ABC, abstractmethod
from importlib.metadata import version
from typing import Dict

from hume._common.api_type import ApiType


class ClientBase(ABC):
    """Base class for Hume API clients."""

    def __init__(
        self,
        api_key: str,
        _api_version: str = "v0",
        _api_http_base_url: str = "https://api.hume.ai",
        _api_ws_base_uri: str = "wss://api.hume.ai",
    ):
        """Construct a new Hume API client.

        Args:
            api_key (str): Hume API key.
        """
        self._api_key = api_key
        self._api_version = _api_version
        self._api_http_base_url = _api_http_base_url
        self._api_ws_base_uri = _api_ws_base_uri

    @classmethod
    @abstractmethod
    def get_api_type(cls) -> ApiType:
        """Get the ApiType of the client.

        Returns:
            ApiType: API type of the client.
        """

    def _get_client_headers(self) -> Dict[str, str]:
        package_version = version("hume")
        return {
            "X-Hume-Api-Key": self._api_key,
            "X-Hume-Client-Name": "python_sdk",
            "X-Hume-Client-Version": package_version,
        }

    def _construct_endpoint(self, path: str) -> str:
        api_type = self.get_api_type()
        base = self._api_ws_base_uri if api_type == ApiType.STREAM else self._api_http_base_url
        return f"{base}/{self._api_version}/{api_type.value}/{path}"



"""Configuration for the facial expression model."""
from dataclasses import dataclass

from hume.models import ModelType
from hume.models.config.model_config_base import ModelConfigBase


@dataclass
class FacemeshConfig(ModelConfigBase["FacemeshConfig"]):
    """Configuration for the facemesh model."""

    @classmethod
    def get_model_type(cls) -> ModelType:
        """Get the configuration model type.

        Returns:
            ModelType: Model type.
        """
        return ModelType.FACEMESH



"""Model configuration utilities."""
from typing import Any, Dict, List, Type

from hume.error.hume_client_exception import HumeClientException
from hume.models import ModelType
from hume.models.config import (BurstConfig, FaceConfig, FacemeshConfig, LanguageConfig, ModelConfigBase, NerConfig,
                                ProsodyConfig)


def config_from_model_type(model_type: ModelType) -> Type[ModelConfigBase]:
    """Get the configuration type for the given model type.

    Args:
        model_type (ModelType): Model type of configuration.

    Returns:
        Type[ModelConfigBase]: Class of configuration for the given model type.
    """
    if model_type == ModelType.BURST:
        return BurstConfig
    if model_type == ModelType.FACE:
        return FaceConfig
    if model_type == ModelType.FACEMESH:
        return FacemeshConfig
    if model_type == ModelType.LANGUAGE:
        return LanguageConfig
    if model_type == ModelType.NER:
        return NerConfig
    if model_type == ModelType.PROSODY:
        return ProsodyConfig
    raise HumeClientException(f"Unknown model type {model_type}")


def serialize_configs(configs: List[ModelConfigBase]) -> Dict[str, Dict[str, Any]]:
    """Convert a list of configs into a dict from model name to serialized model config.

    Args:
        configs (List[ModelConfigBase]): List of configuration objects.

    Returns:
        Dict[str, Dict[str, Any]]: Dictionary of serialized model configurations.
    """
    configs_dict = {}
    for config in configs:
        model_type = config.get_model_type()
        model_name = model_type.value
        configs_dict[model_name] = config.to_dict()
    return configs_dict


def deserialize_configs(configs_dict: Dict[str, Dict[str, Any]]) -> List[ModelConfigBase]:
    """Convert a dict of serialized model configs into a list of config objects.

    Args:
        configs_dict (Dict[str, Dict[str, Any]]): Dictionary of serialized model configurations.

    Returns:
        List[ModelConfigBase]: List of deserialized configuration objects.
    """
    configs = []
    for model_name, config_dict in configs_dict.items():
        model_type = ModelType.from_str(model_name)
        config = config_from_model_type(model_type).from_dict(config_dict)
        configs.append(config)
    return configs



"""Batch job."""
import json
from pathlib import Path
from typing import TYPE_CHECKING, Any, Union

from hume._batch.batch_job_details import BatchJobDetails
from hume._batch.batch_job_status import BatchJobStatus
from hume._common.retry_utils import retry, RetryIterError
from hume.error.hume_client_exception import HumeClientException

if TYPE_CHECKING:
    from hume._batch.hume_batch_client import HumeBatchClient


class BatchJob:
    """Batch job."""

    TIMEOUT_MESSAGE = ("Connection to API has been terminated after {}s, but your job will continue to run. "
                       "Get a reference to your job with `client.get_job('{}')` at any time.")

    def __init__(self, client: "HumeBatchClient", job_id: str):
        """Construct a BatchJob.

        Args:
            client (HumeBatchClient): HumeBatchClient instance.
            job_id (str): Job ID.
        """
        self._client = client
        self.id = job_id

    def get_status(self) -> BatchJobStatus:
        """Get the status of the job.

        Returns:
            BatchJobStatus: The status of the `BatchJob`.
        """
        return self.get_details().state.status

    def get_predictions(self) -> Any:
        """Get `BatchJob` predictions.

        Returns:
            Any: Predictions for the `BatchJob`.
        """
        return self._client.get_job_predictions(self.id)

    def download_predictions(self, filepath: Union[str, Path]) -> None:
        """Download `BatchJob` predictions file.

        Args:
            filepath (Union[str, Path]): Filepath where predictions will be downloaded.
        """
        predictions = self.get_predictions()
        with Path(filepath).open("w") as f:
            json.dump(predictions, f)

    def download_artifacts(self, filepath: Union[str, Path]) -> None:
        """Download `BatchJob` artifacts zip file.

        Args:
            filepath (Optional[Union[str, Path]]): Filepath where artifacts will be downloaded.
        """
        self._client.download_job_artifacts(self.id, filepath)

    def get_details(self) -> BatchJobDetails:
        """Get details for the BatchJob.

        Note that the details for a job may be fetched before the job has completed.
        You may want to use `job.await_complete()` which will wait for the job to
        reach a terminal state before returning.

        Returns:
            BatchJobDetails: Details for the `BatchJob`.
        """
        return self._client.get_job_details(self.id)

    def await_complete(self, timeout: int = 300, raise_on_failed: bool = False) -> BatchJobDetails:
        """Block until the job has reached a terminal status.

        Args:
            timeout (int): Maximum time in seconds to await. If the timeout is reached
                before the job reaches a terminal state the job will continue to be processed,
                but a `HumeClientException` will be raised to the caller of `await_complete`.
            raise_on_failed (bool): If set to True and job fails an exception will be raised.

        Raises:
            ValueError: If the timeout is not valid.
            HumeClientException: If the `BatchJob` has not reached a terminal state within
                the specified timeout. Also can be raised if `raise_on_failed` is set and
                the job reaches a `FAILED` terminal state.

        Returns:
            BatchJobDetails: Details for the `BatchJob`.
        """
        if timeout < 1:
            raise ValueError("timeout must be at least 1 second")

        # pylint: disable=unused-argument
        @retry(timeout_message=self.TIMEOUT_MESSAGE.format(timeout, self.id))
        def _await_complete(timeout: int = timeout) -> BatchJobDetails:
            details = self._client.get_job_details(self.id)
            if not BatchJobStatus.is_terminal(details.state.status):
                raise RetryIterError
            if raise_on_failed and details.state.status == BatchJobStatus.FAILED:
                raise HumeClientException(f"BatchJob {self.id} failed.")
            return details

        return _await_complete(timeout=timeout)

    def __repr__(self) -> str:
        """Get the string representation of the `BatchJob`.

        Returns:
            The the string representation of the `BatchJob`.
        """
        return f'Job(id="{self.id}")'



"""Abstract base class for model configurations."""
from abc import abstractmethod, ABC
from dataclasses import dataclass
from typing import Generic, TypeVar
from hume._common.config_base import ConfigBase

from hume.models import ModelType

T = TypeVar("T")  # Type for subclasses of ModelConfigBase


@dataclass
class ModelConfigBase(ConfigBase["ModelConfigBase"], ABC, Generic[T]):
    """Abstract base class for model configurations."""

    @classmethod
    @abstractmethod
    def get_model_type(cls) -> ModelType:
        """Get the configuration model type.

        Returns:
            ModelType: Model type.
        """
